@@ -13,7 +13,7 @@ package auth
 
 import (
 	"github.com/pkg/errors"
-	extensions "k8s.io/api/extensions/v1beta1"
+	networking "k8s.io/api/networking/v1"
 
 	"github.com/open-cluster-management/management-ingress/pkg/ingress"
 	"github.com/open-cluster-management/management-ingress/pkg/ingress/annotations/parser"
@@ -31,7 +31,7 @@ func NewParser(r resolver.Resolver) parser.IngressAnnotation {
 
 // Parse parses the annotations contained in the ingress
 // rule used to indicate if the upstream servers should use SSL
-func (a at) Parse(ing *extensions.Ingress) (interface{}, error) {
+func (a at) Parse(ing *networking.Ingress) (interface{}, error) {
 	ca, _ := parser.GetStringAnnotation("auth-type", ing)
 	if ca != ingress.IDToken && ca != ingress.AccessToken {
 		return "", errors.Errorf("Auth type %v is not supported", ca)
