@@ -15,7 +15,7 @@ import (
 	"fmt"
 
 	"github.com/pkg/errors"
-	extensions "k8s.io/api/extensions/v1beta1"
+	networking "k8s.io/api/networking/v1"
 
 	"github.com/open-cluster-management/management-ingress/pkg/ingress/annotations/parser"
 	"github.com/open-cluster-management/management-ingress/pkg/ingress/resolver"
@@ -39,7 +39,7 @@ func NewParser(r resolver.Resolver) parser.IngressAnnotation {
 
 // Parse parses the annotations contained in the ingress
 // rule used to indicate if the upstream servers should use SSL
-func (a su) Parse(ing *extensions.Ingress) (interface{}, error) {
+func (a su) Parse(ing *networking.Ingress) (interface{}, error) {
 	s, _ := parser.GetBoolAnnotation("secure-backends", ing)
 	ca, _ := parser.GetStringAnnotation("secure-verify-ca-secret", ing)
 	clientca, _ := parser.GetStringAnnotation("secure-client-ca-secret", ing)
